@@ -12,10 +12,13 @@ from rich.console import Console
 from rich.table import Table
 
 from crashstats_tools.utils import (
+    escape_whitespace,
     DEFAULT_HOST,
     http_get,
     INFINITY,
+    MissingField,
     parse_args,
+    tableize_markdown,
     tableize_tab,
 )
 
@@ -23,17 +26,6 @@ from crashstats_tools.utils import (
 MAX_PAGE = 1000
 
 
-TO_CLEAN = [("\t", "\\t"), ("\r", "\\r"), ("\n", "\\n")]
-
-
-def clean_whitespace(text):
-    text = text or ""
-
-    for s, replace in TO_CLEAN:
-        text = text.replace(s, replace)
-    return text
-
-
 def fetch_supersearch(
     console, host, params, num_results, api_token=None, verbose=False
 ):
@@ -95,9 +87,9 @@ def extract_supersearch_params(url):
     parsed = urlparse(url)
     params = parse_qs(parsed.query)
 
+    # Remove any aggs
+    aggs_keys = ("_facets", "_aggs", "_histogram", "_cardinality")
     for key in list(params.keys()):
-        # Remove any aggs
-        aggs_keys = ("_facets", "_aggs", "_histogram", "_cardinality")
         if key.startswith(aggs_keys):
             del params[key]
 
@@ -130,9 +122,9 @@ def extract_supersearch_params(url):
 @click.option(
     "--format",
     "format_type",
-    default="table",
+    default="tab",
     show_default=True,
-    type=click.Choice(["table", "tab", "json"], case_sensitive=False),
+    type=click.Choice(["table", "tab", "json", "markdown"], case_sensitive=False),
     help="format to print output",
 )
 @click.option(
@@ -201,6 +193,8 @@ def supersearch(ctx, host, supersearch_url, num, headers, format_type, verbose,
 
     https://crash-stats.mozilla.org/documentation/memory_dump_access/
     """
+    host = host.rstrip("/")
+
     if not color:
         console = Console(color_system=None)
     else:
@@ -211,8 +205,6 @@ def supersearch(ctx, host, supersearch_url, num, headers, format_type, verbose,
         # requests which lets us see urls.
         logging.basicConfig(level=logging.DEBUG)
 
-    host = host.rstrip("/")
-
     # Start with params from --url value or product=Firefox
     if supersearch_url:
         params = extract_supersearch_params(supersearch_url)
@@ -221,6 +213,8 @@ def supersearch(ctx, host, supersearch_url, num, headers, format_type, verbose,
 
     params.update(parse_args(ctx.args))
 
+    params["_facets_size"] = 0
+
     if "_columns" not in params:
         params["_columns"] = ["uuid"]
 
@@ -241,12 +235,6 @@ def supersearch(ctx, host, supersearch_url, num, headers, format_type, verbose,
                 "num", 'num needs to be an integer or "all"', ctx=ctx
             )
 
-    console = Console()
-    if not console.is_terminal and format_type == "table":
-        # Switch to tab if stdout is not a terminal. This makes it more
-        # convenient for piping supersearch output
-        format_type = "tab"
-
     if verbose:
         console.print(f"Params: {params}")
 
@@ -266,7 +254,6 @@ def supersearch(ctx, host, supersearch_url, num, headers, format_type, verbose,
                 + "information.[/yellow]"
             )
 
-    records = []
     hits = fetch_supersearch(
         console=console,
         host=host,
@@ -275,28 +262,58 @@ def supersearch(ctx, host, supersearch_url, num, headers, format_type, verbose,
         api_token=api_token,
         verbose=verbose,
     )
-    for hit in hits:
-        records.append(
-            {field: clean_whitespace(hit[field]) for field in params["_columns"]}
-        )
 
     if format_type == "table":
-        table = Table(show_edge=False)
+        table = Table(show_edge=False, show_header=headers)
         for column in params["_columns"]:
             table.add_column(column, justify="left")
-        rows = [[item[field] for field in params["_columns"]] for item in records]
-        for row in rows:
-            table.add_row(*row)
+
+        for hit_i, hit in enumerate(hits):
+            if hit_i == 0:
+                for field in params["_columns"]:
+                    if field not in hit:
+                        raise click.UsageError(f"{field}: no data")
+
+            table.add_row(
+                *[escape_whitespace(hit[field]) for field in params["_columns"]]
+            )
 
         console.print(table)
 
     elif format_type == "tab":
-        rows = [[item[field] for field in params["_columns"]] for item in records]
-        console.print(
-            tableize_tab(headers=params["_columns"], rows=rows, show_headers=headers)
-        )
+        try:
+            for line in tableize_tab(
+                params["_columns"], data=hits, show_headers=headers
+            ):
+                # NOTE(willkg): we don't use console.print here because rich will do fancy
+                # things like wrapping and fixing tabs we don't want that
+                click.echo(line)
+        except MissingField as exc:
+            raise click.UsageError(f"{exc.args[0]}: no data")
+
+    elif format_type == "markdown":
+        try:
+            for line in tableize_markdown(params["_columns"], data=hits):
+                # NOTE(willkg): we don't use console.print here because rich will do fancy
+                # things like wrapping and fixing tabs we don't want that
+                click.echo(line)
+        except MissingField as exc:
+            raise click.UsageError(f"{exc.args[0]}: no data")
 
     elif format_type == "json":
+        records = []
+        for hit_i, hit in enumerate(hits):
+            if hit_i == 0:
+                for field in params["_columns"]:
+                    if field not in hit:
+                        raise click.UsageError(f"{field}: no data")
+
+            records.append(
+                {
+                    field: escape_whitespace(hit.get(field, "<no data>"))
+                    for field in params["_columns"]
+                }
+            )
         console.print_json(json.dumps(records))
 
 
