@@ -7,7 +7,6 @@ import functools
 import json
 import logging
 import os
-import re
 from urllib.parse import urlparse, parse_qs
 
 import click
@@ -18,16 +17,12 @@ from crashstats_tools.utils import (
     DEFAULT_HOST,
     http_get,
     parse_args,
+    parse_relative_date,
     tableize_markdown,
     tableize_tab,
 )
 
 
-RELATIVE_RE = re.compile(r"(\d+)([hdwm])", re.IGNORECASE)
-
-DEFAULT_NOW = datetime.datetime.now().strftime("%Y-%m-%d")
-
-
 @functools.total_ordering
 class AlwaysFirst:
     def __eq__(self, other):
@@ -51,28 +46,19 @@ def now():
     return datetime.datetime.now()
 
 
-def parse_relative_date(text):
-    """Takes a relative date specification and returns a timedelta."""
-    parsed = RELATIVE_RE.match(text)
-    if parsed is None:
-        raise click.UsageError(f"'{text}' is not a valid relative date.")
-
-    count = int(parsed.group(1))
-    unit = parsed.group(2)
-
-    unit_to_arg = {"h": "hours", "d": "days", "w": "weeks"}
-    return datetime.timedelta(**{unit_to_arg[unit]: count})
-
-
 def generate_periods(period, start_date, end_date):
     start_point = datetime.datetime.strptime(start_date, "%Y-%m-%d")
     end_point = datetime.datetime.strptime(end_date, "%Y-%m-%d")
 
     if period == "daily":
         delta = datetime.timedelta(days=1)
+
     elif period == "hourly":
         delta = datetime.timedelta(hours=1)
 
+    elif period == "weekly":
+        delta = datetime.timedelta(days=7)
+
     while start_point <= end_point:
         next_end_point = start_point + delta
         yield start_point.strftime("%Y-%m-%d %H:%M:%S"), next_end_point.strftime(
@@ -108,13 +94,13 @@ def fetch_supersearch_facets(console, host, params, api_token=None, verbose=Fals
 
 
 def extract_supersearch_params(url):
-    """Parses out params from the query string and drops any aggs-related ones."""
+    """Parses out params from the query string and drops any search-related ones."""
     parsed = urlparse(url)
     params = parse_qs(parsed.query)
 
+    # Remove search things
+    aggs_keys = ("_columns", "_results_number", "_results_offset")
     for key in list(params.keys()):
-        # Remove any aggs
-        aggs_keys = ("_facets", "_aggs", "_histogram", "_cardinality")
         if key.startswith(aggs_keys):
             del params[key]
 
@@ -133,9 +119,9 @@ def extract_supersearch_params(url):
 )
 @click.option(
     "--end-date",
-    default=DEFAULT_NOW,
+    default=None,
     show_default=True,
-    help="end date for range; YYYY-MM-DD format",
+    help="end date for range; YYYY-MM-DD format; defaults to today",
 )
 @click.option(
     "--relative-range", default="7d", help="relative range ending on end-date"
@@ -144,7 +130,7 @@ def extract_supersearch_params(url):
     "--period",
     default="none",
     show_default=True,
-    type=click.Choice(["none", "daily", "hourly"], case_sensitive=False),
+    type=click.Choice(["none", "daily", "hourly", "weekly"], case_sensitive=False),
     help="period to facet on to get count/period",
 )
 @click.option(
@@ -198,7 +184,8 @@ def supersearchfacet(
     Make sure to use single quotes when specifying values so that your shell doesn't
     expand variables.
 
-    You need to specify at least one facet using "--_facets".
+    You can only specify one facet using "--_facets". If you don't specify one,
+    it defaults to "signature".
 
     By default, returned data is a tab-delimited table. Tabs and newlines in
     output is escaped. Use "--format" to specify a different output format.
@@ -228,24 +215,24 @@ def supersearchfacet(
 
     https://crash-stats.mozilla.org/documentation/memory_dump_access/
     """
+    host = host.rstrip("/")
+
     if not color:
         console = Console(color_system=None)
     else:
         console = Console()
 
+    if end_date is None:
+        end_date = datetime.datetime.now().strftime("%Y-%m-%d")
+
     # Require at least one facet specified.
     parsed_args = parse_args(ctx.args)
-    if "_facets" not in parsed_args:
-        raise click.ClickException("At least one _facets must be specified.")
-        ctx.exit(1)
 
     if verbose:
         # Set logging to DEBUG because this picks up debug logging from
         # requests which lets us see urls.
         logging.basicConfig(level=logging.DEBUG)
 
-    host = host.rstrip("/")
-
     # Start with params from --url value or product=Firefox
     if supersearch_url:
         params = extract_supersearch_params(supersearch_url)
@@ -254,6 +241,13 @@ def supersearchfacet(
 
     params.update(parsed_args)
 
+    if "_facets" not in params:
+        params["_facets"] = ["signature"]
+
+    if len(params["_facets"]) > 1:
+        raise click.UsageError("One '_facets' must be specified.")
+        ctx.exit(1)
+
     # Sort out API token existence
     api_token = os.environ.get("CRASHSTATS_API_TOKEN")
     if verbose:
@@ -270,16 +264,22 @@ def supersearchfacet(
                 + "information.[/yellow]"
             )
 
-    if not start_date:
-        range_timedelta = parse_relative_date(relative_range)
+    if "date" not in params and not start_date:
+        try:
+            range_timedelta = parse_relative_date(relative_range)
+        except ValueError as ve:
+            raise click.UsageError(ve.msg)
+
         start_date = (
             datetime.datetime.strptime(end_date, "%Y-%m-%d") - range_timedelta
         ).strftime("%Y-%m-%d")
 
-    # If there's no count/period, then we can do a single facet and print the
-    # data out and we're done.
-    if period == "none":
-        params.update({"date": [">=%s" % start_date, "<%s" % end_date]})
+    # If there's no count/period or the user used a supersearch-url that
+    # specifies a date, then we can do a single facet and print the data out
+    # and we're done
+    if period == "none" or "date" in params:
+        if "date" not in params:
+            params.update({"date": [">=%s" % start_date, "<%s" % end_date]})
 
         if verbose:
             console.print(f"Params: {params}")
@@ -292,51 +292,60 @@ def supersearchfacet(
             verbose=verbose,
         )
 
-        if format_type == "json":
-            console.print_json(json.dumps(facet_data))
-            return
+        facet_name = params["_facets"][0]
 
         remaining = facet_data["total"]
         facets = facet_data["facets"]
-        for facet_name in params.get("_facets", facets.keys()):
-            if facet_name not in facets:
-                continue
 
-            headers = [facet_name, "count"]
-            facet_item_data = facets[facet_name]
-            rows = [
-                (item["term"], item["count"])
-                for item in sorted(
-                    facet_item_data, key=lambda x: x["count"], reverse=True
-                )
-            ]
-            remaining -= sum([item["count"] for item in facet_item_data])
+        if facet_name not in facets:
+            raise click.UsageError(f"{facet_name}: no data")
 
-            if remaining:
-                rows.append(("--", remaining))
+        headers = [facet_name, "count"]
+        facet_item_data = facets[facet_name]
+        records = [
+            {facet_name: item["term"], "count": item["count"]}
+            for item in sorted(facet_item_data, key=lambda x: x["count"], reverse=True)
+        ]
+        remaining -= sum([item["count"] for item in facet_item_data])
+
+        if remaining:
+            records.append({facet_name: "--", "count": remaining})
 
         if format_type == "table":
             table = Table(show_edge=False)
             for column in headers:
                 table.add_column(column, justify="left")
-            for row in rows:
-                table.add_row(*[str(cell) for cell in row])
+
+            for item in records:
+                table.add_row(*[str(item[field]) for field in headers])
             console.print(table)
 
         elif format_type == "tab":
-            console.print(tableize_tab(headers=headers, rows=rows))
-        else:
-            console.print(tableize_markdown(headers=headers, rows=rows))
+            # NOTE(willkg): We need to use click.echo because console.print
+            # does rich fancy-stuff with the output
+            for line in tableize_tab(headers=headers, data=records):
+                click.echo(line)
+
+        elif format_type == "markdown":
+            for line in tableize_markdown(headers=headers, data=records):
+                click.echo(line)
+
+        elif format_type == "json":
+            console.print_json(json.dumps(records))
+
         return
 
     # If it is in count/period mode, then we have to do one facet for each
     # period and compose the results.
 
-    # Figure out what facets we're doing
-    facet_names = params.get("_facets", ["signature"])
+    # Figure out what facet we're doing
+    facet_name = params["_facets"][0]
 
     # Map of facet_name -> (map of date -> (map of value -> count))
-    facet_tables = {facet_name: {} for facet_name in facet_names}
+    facet_tables = {facet_name: {}}
+
+    # FIXME(willkg): for "period=weekly", does it make sense to anchor it on
+    # beginning of the week? (sunday or monday)
 
     for day_start, day_end in generate_periods(period, start_date, end_date):
         params.update({"date": [">=%s" % day_start, "<%s" % day_end]})
@@ -355,58 +364,69 @@ def supersearchfacet(
         remaining = facet_data["total"]
         facets = facet_data["facets"]
 
-        for facet_name in facet_names:
-            for item in facets.get(facet_name, []):
-                count = item["count"]
-                facet_tables[facet_name].setdefault(day_start, {})[item["term"]] = count
-                remaining -= count
-            facet_tables[facet_name].setdefault(day_start, {})["--"] = remaining
+        for item in facets.get(facet_name, []):
+            count = item["count"]
+            facet_tables[facet_name].setdefault(day_start, {})[item["term"]] = count
+            remaining -= count
+        facet_tables[facet_name].setdefault(day_start, {})["--"] = remaining
 
     # Normalize the data--make sure table rows have all the values
-    for facet_name in facet_names:
-        values = set()
+    values = set()
 
-        table = facet_tables[facet_name]
-        for date, value_counts in table.items():
-            values = values | set(value_counts.keys())
+    table = facet_tables[facet_name]
+    for date, value_counts in table.items():
+        values = values | set(value_counts.keys())
 
-        for date, value_counts in table.items():
-            missing_values = values - set(value_counts.keys())
-            for missing_value in missing_values:
-                value_counts[missing_value] = 0
+    for date, value_counts in table.items():
+        missing_values = values - set(value_counts.keys())
+        for missing_value in missing_values:
+            value_counts[missing_value] = 0
 
-    # Print out the normalized data
-    if format_type == "json":
-        console.print_json(json.dumps(facet_tables))
-        return
+    table = facet_tables[facet_name]
 
-    for facet_name in facet_names:
-        table = facet_tables[facet_name]
+    if not table:
+        raise click.UsageError(f"{facet_name}: no data")
 
-        if not table:
-            console.print(f"{facet_name}: no data")
-            continue
+    some_date = list(table.keys())[0]
+    headers = ["date"] + sorted(table[some_date].keys(), key=thing_to_key)
+    records = []
+    for date, value_counts in table.items():
+        records.append(
+            {
+                field_name: val
+                for field_name, val in zip(
+                    headers,
+                    [date]
+                    + [
+                        item[1]
+                        for item in sorted(value_counts.items(), key=thing_to_key)
+                    ],
+                )
+            }
+        )
 
-        some_date = list(table.keys())[0]
-        headers = ["date"] + sorted(table[some_date].keys(), key=thing_to_key)
-        rows = []
-        for date, value_counts in table.items():
-            rows.append(
-                [date]
-                + [item[1] for item in sorted(value_counts.items(), key=thing_to_key)]
-            )
+    if format_type == "table":
+        table = Table(show_edge=False)
+        for column in headers:
+            table.add_column(column, justify="left")
+        for item in records:
+            table.add_row(*[str(item[field]) for field in headers])
+        console.print(table)
 
-        if format_type == "table":
-            table = Table(show_edge=False)
-            for column in headers:
-                table.add_column(column, justify="left")
-            for row in rows:
-                table.add_row(*[str(cell) for cell in row])
-            console.print(table)
-        elif format_type == "tab":
-            console.print(tableize_tab(headers=headers, rows=rows))
-        elif format_type == "markdown":
-            console.print(tableize_markdown(headers=headers, rows=rows))
+    elif format_type == "tab":
+        # NOTE(willkg): We need to use click.echo because console.print
+        # does rich fancy-stuff with the output
+        for line in tableize_tab(headers=headers, data=records):
+            click.echo(line)
+
+    elif format_type == "markdown":
+        # NOTE(willkg): We need to use click.echo because console.print
+        # does rich fancy-stuff with the output
+        for line in tableize_markdown(headers=headers, data=records):
+            click.echo(line)
+
+    elif format_type == "json":
+        console.print_json(json.dumps(records))
 
 
 if __name__ == "__main__":
