@@ -6,6 +6,7 @@ import datetime
 from functools import total_ordering
 import json
 import re
+from typing import Any, Dict, Generator, Iterable, List
 from urllib.parse import urlparse
 
 import requests
@@ -16,17 +17,18 @@ from requests.packages.urllib3.util.retry import Retry
 DEFAULT_HOST = "https://crash-stats.mozilla.org"
 
 
-WHITESPACE_TO_CLEAN = [("\t", "\\t"), ("\r", "\\r"), ("\n", "\\n")]
+WHITESPACE_TO_ESCAPE = [("\t", "\\t"), ("\r", "\\r"), ("\n", "\\n")]
 
 
-def clean_whitespace(text):
+def escape_whitespace(text):
+    """Escapes whitespace characters."""
     text = text or ""
-    for s, replace in WHITESPACE_TO_CLEAN:
+    for s, replace in WHITESPACE_TO_ESCAPE:
         text = text.replace(s, replace)
     return text
 
 
-def clean_pipes(text):
+def escape_pipes(text):
     text = text or ""
     return text.replace("|", "\\|")
 
@@ -307,38 +309,72 @@ def parse_crashid(item):
     raise ValueError(f"Not a valid crash id: {item}")
 
 
-def tableize_tab(headers, rows, show_headers=True):
+class MissingField(Exception):
+    pass
+
+
+def tableize_tab(
+    headers: List[str], data: Iterable[Dict[str, Any]], show_headers: bool = True
+) -> Generator[str, None, None]:
     """Generate output for a table using tab delimiters.
 
-    :param list-of-str headers: headers of the table
-    :param list-of-str rows: rows of the table
+    :param headers: headers of the table
+    :param data: rows of the table
 
-    :returns: string
+    :returns: generator of strings
 
     """
-    output = []
-    if show_headers:
-        output.append("\t".join([clean_whitespace(str(item)) for item in headers]))
-    for row in rows:
-        output.append("\t".join([clean_whitespace(str(item)) for item in row]))
-    return "\n".join(output)
+    for item_i, item in enumerate(data):
+        if item_i == 0:
+            for field in headers:
+                if field not in item:
+                    raise MissingField(field)
+            if show_headers:
+                yield "\t".join([escape_whitespace(str(item)) for item in headers])
+
+        row = [escape_whitespace(str(item.get(field, ""))) for field in headers]
+        yield "\t".join(row) or "<no data>"
 
 
-def tableize_markdown(headers, rows, show_headers=True):
+def tableize_markdown(
+    headers: List[str], data: Iterable[Dict[str, Any]], show_headers: bool = True
+) -> Generator[str, None, None]:
     """Generate output for a table using markdown.
 
-    :param list-of-str headers: headers of the table
-    :param list-of-str rows: rows of the table
+    :param headers: headers of the table
+    :param data: rows of the table
 
-    :returns: string
+    :returns: generator of strings
 
     """
-    output = []
-    if show_headers:
-        output.append(" | ".join([str(header) for header in headers]))
-        output.append(" | ".join(["-" * len(str(item)) for item in headers]))
-    for row in rows:
-        output.append(
-            " | ".join([clean_pipes(clean_whitespace(str(item))) for item in row])
-        )
-    return "\n".join(output)
+    for item_i, item in enumerate(data):
+        if item_i == 0:
+            for field in headers:
+                if field not in item:
+                    raise MissingField(field)
+
+            if show_headers:
+                yield " | ".join([str(header) for header in headers])
+                yield " | ".join(["-" * len(str(item)) for item in headers])
+
+        row = [escape_pipes(escape_whitespace(str(item[field]))) for field in headers]
+        yield " | ".join(row) or "<no data>"
+
+
+RELATIVE_RE = re.compile(r"(\d+)([hdw])", re.IGNORECASE)
+
+
+def parse_relative_date(text):
+    """Takes a relative date specification and returns a timedelta."""
+    if not text or not isinstance(text, str):
+        raise ValueError(f"'{text}' is not a valid relative date.")
+
+    parsed = RELATIVE_RE.match(text)
+    if parsed is None:
+        raise ValueError(f"'{text}' is not a valid relative date.")
+
+    count = int(parsed.group(1))
+    unit = parsed.group(2)
+
+    unit_to_arg = {"h": "hours", "d": "days", "w": "weeks"}
+    return datetime.timedelta(**{unit_to_arg[unit]: count})
